@@ -1,11 +1,11 @@
 /*
  * Copyright (c) 2007, to-do. All rights reserved.
  */
+case K_exec: return _global_exec(ex,argc,argv,res);
 case K_getenv: return _global_getenv(ex,argc,argv,res);
 case K_setenv: return _global_setenv(ex,argc,argv,res);
 case K_system: return _global_system(ex,argc,argv,res);
 case K_popen: return _global_popen(ex,argc,argv,res);
-case K_pclose: return _global_pclose(ex,argc,argv,res);
 case K_load: return _global_load(ex,argc,argv,res);
 case K_eval: return _global_eval(ex,argc,argv,res);
 case K_print: return _global_print(ex,argc,argv,res);
@@ -85,6 +85,12 @@ case K_timegm: return _global_timegm(ex,argc,argv,res);
 case K_time: return _global_time(ex,argc,argv,res);
 case K_date: return _global_date(ex,argc,argv,res);
 case K_gmdate: return _global_gmdate(ex,argc,argv,res);
+case K_putheader: return _global_putheader(ex,argc,argv,res);
+case K_setcookie: return _global_setcookie(ex,argc,argv,res);
+case K_sendmail: return _global_sendmail(ex,argc,argv,res);
+case K_popmail: return _global_popmail(ex,argc,argv,res);
+case K_ftpsite: return _global_ftpsite(ex,argc,argv,res);
+case K_httpget: return _global_httpget(ex,argc,argv,res);
 case K_length: return _global_length(ex,argc,argv,res);
 case K_isNull: return _global_isNull(ex,argc,argv,res);
 case K_isBool: return _global_isBool(ex,argc,argv,res);
