@@ -5,38 +5,40 @@
 #define _OP_H_
 
 typedef enum {
-C_void      = 0x01,
-C_addA      = 0x02,
-C_subA      = 0x03,
-C_mulA      = 0x04,
-C_divA      = 0x05,
-C_modA      = 0x06,
-C_powA      = 0x07,
-C_bitandA   = 0x08,
-C_bitorA    = 0x09,
-C_bitsubA   = 0x0A,
-C_bitxorA   = 0x0B,
-C_bitshLA   = 0x0C,
-C_bitshRA   = 0x0D,
-C_andA      = 0x0E,
-C_orA       = 0x0F,
-C_inc       = 0x10,
-C_dec       = 0x11,
-C_echo      = 0x12,
-C_doc       = 0x13,
-C_crlf      = 0x14,
-C_if        = 0x15,
-C_else      = 0x16,
-C_for       = 0x17,
-C_while     = 0x18,
-C_do        = 0x19,
-C_foreach   = 0x1A,
-C_switch    = 0x1B,
-C_default   = 0x1C,
-C_try       = 0x1D,
-C_break     = 0x1E,
-C_continue  = 0x1F,
-C_in        = 0x20,
+C_eof   = -1,
+C_error =  0,
+C_void      = 1,
+C_addA      = 2,
+C_subA      = 3,
+C_mulA      = 4,
+C_divA      = 5,
+C_modA      = 6,
+C_powA      = 7,
+C_bitandA   = 8,
+C_bitorA    = 9,
+C_bitsubA   = 10,
+C_bitxorA   = 11,
+C_bitshLA   = 12,
+C_bitshRA   = 13,
+C_andA      = 14,
+C_orA       = 15,
+C_inc       = 16,
+C_dec       = 17,
+C_echo      = 18,
+C_doc       = 19,
+C_crlf      = 20,
+C_if        = 21,
+C_else      = 22,
+C_for       = 23,
+C_while     = 24,
+C_do        = 25,
+C_foreach   = 26,
+C_switch    = 27,
+C_default   = 28,
+C_try       = 29,
+C_break     = 30,
+C_continue  = 31,
+C_in        = 32,
 C_not       = '!',
 C_string    = '"',
 C_sharp     = '#',
