@@ -70,6 +70,11 @@ ex_class,
 ex_file,
 ex_line,
 ex_names,
+ex_stdin,
+ex_stdout,
+ex_stderr,
+ex_tmpout,
+ex_tmperr,
 ex_frameIn,
 ex_frameOut,
 ex_jump,
@@ -88,17 +93,17 @@ ex_bitnot,
 ex_toBool,
 ex_toChar,
 ex_toInt,
+NULL,
 ex_toFloat,
+NULL,
+ex_bitxor,
 ex_toString,
+ex_regex,
 ex_toNumber,
 ex_typeof,
 ex_delete,
-NULL,
 ex_new,
-NULL,
-ex_bitxor,
 ex_null,
-ex_regex,
 ex_true,
 ex_false,
 ex_this,
@@ -121,11 +126,6 @@ NULL,
 NULL,
 NULL,
 NULL,
-NULL,
-NULL,
-NULL,
-NULL,
-NULL,
 ex_bitor,
 NULL,
 ex_bitsub,
