@@ -70,6 +70,11 @@ static char* op2str[] = {
 "file",
 "line",
 "names",
+"stdin",
+"stdout",
+"stderr",
+"tmpout",
+"tmperr",
 "frameIn",
 "frameOut",
 "jump",
@@ -88,17 +93,17 @@ static char* op2str[] = {
 "toBool",
 "toChar",
 "toInt",
+"sqL",
 "toFloat",
+"sqR",
+"bitxor",
 "toString",
+"regex",
 "toNumber",
 "typeof",
 "delete",
-"sqL",
 "new",
-"sqR",
-"bitxor",
 "null",
-"regex",
 "true",
 "false",
 "this",
@@ -120,11 +125,6 @@ static char* op2str[] = {
 "???",
 "???",
 "???",
-"???",
-"???",
-"???",
-"???",
-"???",
 "cuL",
 "bitor",
 "cuR",
