@@ -17,10 +17,9 @@
 #define LF_LEFT       0x20
 
 /* VALUE FLAG */
-#define VF_OBJECT     0x0001
-#define VF_NEW        0x0002
-#define VF_CONST      0x0004
-#define VF_RETURN     0x0008
+#define VF_OBJECT     0x0002
+#define VF_NEW        0x0004
+#define VF_CONST      0x0008
 #define VF_RESOLV     0x00FF
 #define VF_SYMBOL     0x0100
 #define VF_REFER      0x0200
@@ -160,7 +159,7 @@ static void put_regex(compile_t* cp, char *s, int n)
 	q = *s++;
 	n = 0;
 	while ((s < e) && (*s != q)) {
-		if ((c = *s++) == '\\' && *s) {
+		if ((c = *s++) == '\\' && (*s > 0)) {
 			if (*s == q) c = *s++;
 			else if (*s == '\\') d[n++] = *s++;
 		}
@@ -217,6 +216,11 @@ static void put_doc(compile_t* cp, char *s, int n, int str)
 		}
 	}
 }
+static void put_lineno(compile_t* cp)
+{
+	put_c(cp, C_line);
+	put_ushort(cp, cp->ln);
+}
 
 /************************************************************
  *
@@ -241,7 +245,7 @@ static int seek_doc(compile_t* cp)
 	char *s;
 	int c;
 	s = cp->sc;
-	while ((c = *s++) != 0) {
+	while ((c = *s++) > 0) {
 		if ((c == '<') && (*s=='?') && !islower(*(s + 1))) {
 			break;
 		}
@@ -259,10 +263,10 @@ static int seek_regex(compile_t* cp)
 	int c, q;
 	q = *cp->sc;
 	s = cp->sc + 1;
-	while (((c = *s++) != 0) && (c != q)) {
+	while (((c = *s++) > 0) && (c != q)) {
 		if (c == '\\') c = *s++;
 		if (iscrlf(c)) {
-			if (*s==(c^7)) s++;
+			if (*s == (c ^ 7)) s++;
 			cp->nr++;
 		}
 	}
@@ -278,10 +282,10 @@ static int seek_string(compile_t* cp)
 	int c,q;
 	q = *cp->sc;
 	s = cp->sc + 1;
-	while (((c = *s++) != 0) && (c != q)) {
+	while (((c = *s++) > 0) && (c != q)) {
 		if (c == '\\') c = *s++;
 		if (iscrlf(c)) {
-			if (*s==(c^7)) s++;
+			if (*s == (c ^ 7)) s++;
 			cp->nr++;
 		}
 	}
@@ -346,7 +350,7 @@ static int lex(compile_t* cp, int w)
 {
 	char *s;
 	int c, q;
-	if ((c = cp->lc) != 0) {
+	if ((c = cp->lc) > 0) {  /* > EOF, ERR */
 		cp->lc = 0;
 		return c;
 	}
@@ -356,7 +360,7 @@ static int lex(compile_t* cp, int w)
 	cp->nr = 0;
 	q = 0;
 	s = cp->sc;
-	while ((c = *s++) != 0) {
+	while ((c = *s++) > 0) {
 		if (s > cp->es) {
 			c = 0;
 			break;
@@ -392,9 +396,8 @@ static int lex(compile_t* cp, int w)
 		}
 	}
 	cp->sc = s - 1;
-	if (!c) {
-		/* EOF */
-		return 1;
+	if (c <= 0) {
+		return -1; /* EOF */
 	}
 	else if (iscrlf(c)) {
 		cp->sc = s;
@@ -454,7 +457,7 @@ static int lex(compile_t* cp, int w)
 
 static void unlex(compile_t* cp, op_t c)
 {
-	assert ((c > 1) && (cp->lc == 0));
+	assert ((c > 0) && (cp->lc == 0)); /* > EOF, ERR */
 	cp->lc = c;
 }
 
@@ -619,7 +622,7 @@ static int is_right(op_t c)
 	}
 	return 0;
 }
-static int is_left(op_t c)
+static int is_prefix(op_t c)
 {
 	switch (c) {
 	case C_add: return C_plus;
@@ -770,53 +773,9 @@ static void calc_const(compile_t* cp, int i1, op_t c, int i2)
  *
  ************************************************************/
 static int expr_post(compile_t* cp);
-static int expr_left(compile_t* cp);
-static int expr_next(compile_t* cp, int prev);
-
-static int expr_post(compile_t* cp)
-{
-	op_t c;
-	int n,vf;
-	vf = cp->vf;
-	c = lex(cp, LF_RIGHT); /* get right */
-	if (c == '.') {
-		c = lex(cp, LF_RIGHT);
-		exerrif(cp, c != C_symbol, "bad property");
-		put_c_int(cp, C_symbol, add_symbol(cp, cp->sc, cp->nc));
-		put_c(cp, C_join);
-		cp->vf = VF_REFER;
-		c = expr_post(cp);
-	} else if (c == '[') {
-		c = expr_next(cp, 0);
-		exerrif(cp, c != ']', "bad index");
-		put_c(cp, C_join);
-		cp->vf = VF_REFER;
-		c = expr_post(cp);
-	} else if (c == '(') {
-		exerrif(cp, !(vf & VF_UNRESOLV), "bad call");
-		n = 0;
-		do {
-			c = expr_next(cp, 0);
-			if (!cp->vf) break;
-			n++;
-		} while (c == ',');
-		exerrif(cp, c != ')', "bad brace");
-		put_c_int(cp, C_call, n);
-		cp->vf = (vf == VF_SYMBOL) ? VF_NEW : VF_OBJECT;
-		c = expr_post(cp);
-	} else if (c==C_inc || c==C_dec) {
-		exerrif(cp, !(vf & VF_UNRESOLV), "bad postfix");
-		put_c(cp, C_copy);
-		put_c(cp, C_load);
-		put_c(cp, C_sand);
-		put_c(cp, c);
-		put_c(cp, C_store);
-		put_c(cp, C_pop);
-		cp->vf = VF_OBJECT; /* resolved */
-		c = lex(cp, LF_RIGHT);
-	}
-	return c;
-}
+static int expr_pre(compile_t* cp, int prev);
+static int expr_brace(compile_t* cp, int prev);
+static int stmt_function(compile_t* cp, int prev);
 
 /************************************************************
  *
@@ -914,6 +873,7 @@ static int expr_const(compile_t* cp, int k)
 	}
 	return 0;
 }
+
 static int expr_symbol(compile_t* cp)
 {
 	char *s;
@@ -935,18 +895,66 @@ static int expr_symbol(compile_t* cp)
 	}
 	return c;
 }
-static int expr_left(compile_t* cp)
+
+static int expr_post(compile_t* cp)
+{
+	op_t c;
+	int n,vf;
+	vf = cp->vf;
+	c = lex(cp, LF_RIGHT); /* get right */
+	if (c == '.') {
+		c = lex(cp, LF_RIGHT);
+		exerrif(cp, c != C_symbol, "bad property");
+		put_c_int(cp, C_symbol, add_symbol(cp, cp->sc, cp->nc));
+		put_c(cp, C_join);
+		cp->vf = VF_REFER;
+		c = expr_post(cp);
+	} else if (c == '[') {
+		c = expr_brace(cp, 0);
+		exerrif(cp, c != ']', "bad index");
+		put_c(cp, C_join);
+		cp->vf = VF_REFER;
+		c = expr_post(cp);
+	} else if (c == '(') {
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad arguments");
+		n = 0;
+		do {
+			c = expr_brace(cp, 0);
+			if (!cp->vf) break;
+			n++;
+		} while (c == ',');
+		exerrif(cp, c != ')', "bad arguments");
+		put_c_int(cp, C_call, n);
+		cp->vf = (vf == VF_SYMBOL) ? VF_NEW : VF_OBJECT;
+		c = expr_post(cp);
+	} else if (c==C_inc || c==C_dec) {
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad postfix");
+		put_c(cp, C_copy);
+		put_c(cp, C_load);
+		put_c(cp, C_sand);
+		put_c(cp, c);
+		put_c(cp, C_store);
+		put_c(cp, C_pop);
+		cp->vf = VF_OBJECT; /* resolved */
+		c = lex(cp, LF_RIGHT);
+	}
+	return c;
+}
+
+static int expr_pre(compile_t* cp, int prev)
 {
 	op_t c;
 	int n;
+	
 	c = lex(cp, 0);
 	if (c == '(') {
-		exerrif(cp,  cp->vf != 0, "bad value");
-		exerrif(cp,  (c = lex(cp, 0)) <= 1, "no value");
+		exerrif(cp,  cp->vf != 0, "bad brace");
+		c = lex(cp, 0);
+		exerrif(cp, c <= 0, "bad brace"); /* <= EOF, ERR */
 		if (is_typecast(c)) {
-			exerrif(cp,  lex(cp, 0) != ')', "bad cast");
-			n = expr_left(cp);
-			exerrif(cp,  !cp->vf , "bad left for cast");
+			exerrif(cp, lex(cp, 0) != ')', "bad cast");
+			n = expr_pre(cp, 0);
+			exerrif(cp,  !cp->vf , "bad cast");
 			if (cp->vf & VF_UNRESOLV) {
 				put_c(cp, C_load);
 			}
@@ -956,7 +964,7 @@ static int expr_left(compile_t* cp)
 		} else {
 			if (c != ')') {
 				unlex(cp, c);
-				c = expr_next(cp, -1);
+				c = expr_brace(cp, -1);
 			}
 			exerrif(cp, c != ')', "bad sub-expr");
 			exerrif(cp, !cp->vf, "bad sub-expr");
@@ -964,8 +972,8 @@ static int expr_left(compile_t* cp)
 		}
 	} else if (c==C_inc || c==C_dec) {
 		exerrif(cp, cp->vf != 0, "bad prefix");
-		n = expr_left(cp);
-		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad left for prefix");
+		n = expr_pre(cp, 0);
+		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad prefix");
 		put_c(cp, C_copy);
 		put_c(cp, C_load);
 		put_c(cp, c);
@@ -973,44 +981,28 @@ static int expr_left(compile_t* cp)
 		put_c(cp, C_load);
 		cp->vf = VF_OBJECT;
 		c = n;
-	} else if (c == C_delete) {
-		exerrif(cp, cp->vf != 0, "bad delete");
-		n = expr_left(cp);
-		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad left for delete");
-		put_c(cp, c);
-		cp->vf = VF_OBJECT;
-		c = n;
-	} else if (c == C_new) {
-		exerrif(cp, cp->vf != 0, "bad new");
-		n = expr_left(cp);
-		exerrif(cp, cp->vf != VF_NEW, "bad left for new");
-		put_c(cp, c);
-		cp->vf = VF_OBJECT;
-		c = n;
-	} else if (c == C_bitand) { /* reference */
+	} else if (c == C_bitand) { /* `&` reference */
 		exerrif(cp, cp->vf != 0, "bad reference");
-		n = expr_left(cp);
-		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad left for reference");
+		c = expr_pre(cp, 0);
+		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad reference");
 		cp->vf = VF_OBJECT;
-		c = n;
-	} else if ((n = is_left(c)) > 0) {
-		c = n;
+	} else if ((n = is_prefix(c)) > 0) {
 		exerrif(cp, cp->vf != 0, "bad prefix");
-		n = expr_left(cp);
+		c = expr_pre(cp, 0);
+		exerrif(cp, !cp->vf, "bad prefix");
 		if (cp->vf & VF_UNRESOLV) {
 			put_c(cp, C_load);
 		}
-		put_c(cp, c);
+		put_c(cp, n);
 		cp->vf = VF_OBJECT;
-		c = n;
 	} else if (c=='[') {
 		n = 0;
 		do {
-			c = expr_next(cp, 0);
+			c = expr_brace(cp, 0);
 			if (!cp->vf) break;
 			n++;
 		} while (c == ',');
-		exerrif(cp, c != ']', "bad const array");
+		exerrif(cp, c != ']', "bad const-array");
 		put_c_int(cp, C_array, n);
 		cp->vf = VF_OBJECT;
 		c = lex(cp, LF_RIGHT); /* get right */
@@ -1026,11 +1018,11 @@ static int expr_left(compile_t* cp)
 				break;
 			}
 			c = lex(cp, 0);
-			exerrif(cp, c != ':', "bad item");
-			c = expr_next(cp, 0);
+			exerrif(cp, c != ':', "bad const-object");
+			c = expr_brace(cp, 0);
 			n++;
 		} while (c == ',');
-		exerrif(cp, c != '}', "bad const object");
+		exerrif(cp, c != '}', "bad const-object");
 		put_c_int(cp, C_object, n);
 		cp->vf = VF_OBJECT;
 		c = lex(cp, LF_RIGHT); /* get right */
@@ -1052,16 +1044,107 @@ static int expr_left(compile_t* cp)
 		put_float(cp, strtod(cp->sc, NULL));
 		cp->vf = VF_CONST;
 		c = lex(cp, LF_RIGHT);
+	} else if (c == C_doc) {
+		put_doc(cp, cp->sc, cp->nc, 1);
+		cp->vf = VF_CONST;
+		c = ';';
 	} else if (c == C_null
 	|| c == C_true
 	|| c == C_false) {
 		put_c(cp, c);
 		cp->vf = VF_CONST;
 		c = lex(cp, LF_RIGHT);
-	} else if (c == C_doc) {
-		put_doc(cp, cp->sc, cp->nc, 1);
-		cp->vf = VF_CONST;
-		c = ';';
+	} else if (c == C_break) {
+		exerrif(cp, cp->vf != 0, "bad break");
+		exerrif(cp, !(cp->fp->af & AF_BREAK), "bad break");
+		put_c_int(cp, C_goto, K_break);
+		c = lex(cp, 0);
+		exerrif(cp, c != ';', "bad break");
+		cp->vf = 0;
+	} else if (c == C_continue) {
+		exerrif(cp, cp->vf != 0, "bad continue");
+		exerrif(cp, !(cp->fp->af & AF_CONTINUE), "bad continue");
+		put_c_int(cp, C_goto, K_continue);
+		c = lex(cp, 0);
+		exerrif(cp, c != ';', "bad continue");
+		cp->vf = 0;
+	} else if (c == C_goto) {
+		exerrif(cp, cp->vf != 0, "bad goto");
+		c = lex(cp, 0);
+		exerrif(cp, c != C_symbol, "bad goto");
+		n = add_symbol(cp, cp->sc, cp->nc);
+		put_c_int(cp, C_goto, n);
+		c = lex(cp, 0);
+		exerrif(cp, c != ';', "bad goto");
+		cp->vf = 0;
+	} else if (c == C_return) {
+		exerrif(cp, cp->vf != 0, "bad return");
+		exerrif(cp, !(cp->fp->af & AF_RETURN), "bad return");
+		c = lex(cp, 0);
+		exerrif(cp, c <= 0, "bad return"); /* <= EOF, ERR */
+		if (c == ';') {
+			put_c(cp, C_null);
+		} else {
+			unlex(cp, c);
+			c = expr_brace(cp, 0);
+		}
+		exerrif(cp, c != ';', "bad return");
+		put_c(cp, C_return);
+		cp->vf = 0;
+	} else if (c == C_throw) {
+		exerrif(cp, cp->vf != 0, "bad throw");
+		c = lex(cp, 0);
+		exerrif(cp, c <= 0, "bad throw"); /* <= EOF, ERR */
+		if (c == ';') {
+			put_c(cp, C_null);
+		} else {
+			unlex(cp, c);
+			c = expr_brace(cp, 0);
+		}
+		exerrif(cp, c != ';', "bad throw");
+		put_c(cp, C_throw);
+		cp->vf = 0;
+	} else if (c == C_exit) {
+		exerrif(cp, cp->vf != 0, "bad exit");
+		c = lex(cp, 0);
+		exerrif(cp, c <= 0, "bad exit"); /* <= EOF, ERR */
+		if (c == ';') {
+			put_c(cp, C_null);
+		} else {
+			unlex(cp, c);
+			c = expr_brace(cp, 0);
+		}
+		exerrif(cp, c != ';', "bad exit");
+		put_c(cp, C_exit);
+		cp->vf = 0;
+	} else if (c == C_function) {
+		exerrif(cp, cp->vf != 0, "bad function");
+		c = stmt_function(cp, 1);
+		exerrif(cp, c != ';', "bad function");
+		cp->vf = VF_OBJECT;
+		c = lex(cp, 0);
+	} else if (c == C_new) {
+		exerrif(cp, cp->vf != 0, "bad new");
+		c = expr_pre(cp, 0);
+		exerrif(cp, cp->vf != VF_NEW, "bad new");
+		put_c(cp, C_new);
+		cp->vf = VF_OBJECT;
+	} else if (c == C_delete) {
+		exerrif(cp, cp->vf != 0, "bad delete");
+		c = expr_pre(cp, 0);
+		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad delete");
+		put_c(cp, C_delete);
+		cp->vf = VF_OBJECT;
+	} else if (c == C_var) {
+		exerrif(cp, cp->vf != 0, "bad var");
+		c = lex(cp, 0);
+		exerrif(cp, c != C_symbol, "bad var");
+		n = add_symbol(cp, cp->sc, cp->nc);
+		add_local(cp, C_var, n, 0);
+		c = lex(cp, 0);
+		exerrif(cp, c != '=', "bad var");
+		put_c_int(cp, C_symbol, n);
+		cp->vf = VF_SYMBOL;
 	} else if (c == C_this) {
 		exerrif(cp, !(cp->fp->af & AF_THIS), "bad this");
 		put_c(cp, c);
@@ -1069,77 +1152,25 @@ static int expr_left(compile_t* cp)
 		c = expr_post(cp);
 	} else if (c == C_symbol) {
 		c = expr_symbol(cp);
-	} else if (c == C_break) {
-		exerrif(cp, !(cp->fp->af & AF_BREAK), "bad break");
-		put_c_int(cp, C_goto, K_break);
-		c = lex(cp, 0);
-		exerrif(cp, c != ';', "bad break");
-		cp->vf = VF_RETURN;
-	} else if (c == C_continue) {
-		exerrif(cp, !(cp->fp->af & AF_CONTINUE), "bad continue");
-		put_c_int(cp, C_goto, K_continue);
-		c = lex(cp, 0);
-		exerrif(cp, c != ';', "bad continue");
-		cp->vf = VF_RETURN;
-	} else if (c == C_return) {
-		exerrif(cp, !(cp->fp->af & AF_RETURN), "bad return");
-		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad return");
-		if (c == ';') {
-			put_c(cp, C_null);
-		} else {
-			unlex(cp, c);
-			c = expr_left(cp);
-		}
-		exerrif(cp, c != ';', "bad return");
-		put_c(cp, C_return);
-		cp->vf = VF_RETURN;
-	} else if (c == C_throw) {
-		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad throw");
-		if (c == ';') {
-			put_c(cp, C_null);
-		} else {
-			unlex(cp, c);
-			c = expr_left(cp);
-		}
-		exerrif(cp, c != ';', "bad throw");
-		put_c(cp, C_throw);
-		cp->vf = VF_RETURN;
-	} else if (c == C_exit) {
-		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad exit");
-		if (c == ';') {
-			put_c(cp, C_null);
-		} else {
-			unlex(cp, c);
-			c = expr_left(cp);
-		}
-		exerrif(cp, c != ';', "bad exit");
-		put_c(cp, C_exit);
-		cp->vf = VF_RETURN;
-	} else if (c == C_goto) {
-		c = lex(cp, 0);
-		exerrif(cp, c != C_symbol, "bad label");
-		put_c_int(cp, C_goto, add_symbol(cp, cp->sc, cp->nc));
-		c = lex(cp, 0);
-		exerrif(cp, c != ';', "bad goto");
-		cp->vf = VF_RETURN;
 	}
 	return c;
 }
 
-static int expr_next(compile_t* cp, int prev)
+static int expr_brace(compile_t* cp, int prev)
 {
 	op_t c;
 	int n,j,e,vf;
 	cp->vf = 0;
 	j = cp->bn;
-	c = expr_left(cp);
+	c = expr_pre(cp, 0);
 	vf = cp->vf;
-	if (!vf || (vf == VF_RETURN)) {
+	if (!vf) {
 		return c;
 	}
 	else if (c == '=') {
 		exerrif(cp, !(vf & VF_UNRESOLV), "bad assign");
-		n = expr_next(cp, 0);
+		n = expr_brace(cp, 0);
+		exerrif(cp, !cp->vf, "bad assign");
 		put_c(cp, C_store);
 		c = n;
 	}
@@ -1152,7 +1183,7 @@ static int expr_next(compile_t* cp, int prev)
 		put_c(cp, c);
 		j = seek_int(cp, 0);
 		put_c(cp, C_pop);
-		c = expr_next(cp, 0);
+		c = expr_brace(cp, 0);
 		put_c(cp, C_store);
 		set_offset(cp, j);
 	}
@@ -1160,7 +1191,7 @@ static int expr_next(compile_t* cp, int prev)
 		exerrif(cp, !(vf & VF_UNRESOLV), "bad assign");
 		put_c(cp, C_copy);
 		put_c(cp, C_load);
-		c = expr_next(cp, 0);
+		c = expr_brace(cp, 0);
 		put_c(cp, n);
 		put_c(cp, C_store);
 	}
@@ -1173,7 +1204,7 @@ static int expr_next(compile_t* cp, int prev)
 		put_c(cp, c);
 		j = seek_int(cp, 0);
 		put_c(cp, C_pop);
-		c = expr_next(cp, 0);
+		c = expr_brace(cp, 0);
 		set_offset(cp, j);
 	}
 	else if (c == '?') {
@@ -1183,11 +1214,11 @@ static int expr_next(compile_t* cp, int prev)
 		}
 		put_c(cp, C_and);
 		j = seek_int(cp, 0);
-		c = expr_next(cp, 0);
+		c = expr_brace(cp, 0);
 		e = seek_int(cp, 0);
-		exerrif(cp, c != ':', "bad ?: statement");
+		exerrif(cp, c != ':', "bad `?:` statement");
 		set_offset(cp, j);
-		c = expr_next(cp, 0);
+		c = expr_brace(cp, 0);
 		set_offset(cp, e);
 	}
 	else if ((n = is_right(c)) != 0) {
@@ -1198,7 +1229,7 @@ static int expr_next(compile_t* cp, int prev)
 		if (prev < 0) prev = 0;
 		while (n > prev) {
 			e = cp->bn;
-			n = expr_next(cp, n);
+			n = expr_brace(cp, n);
 			if ((vf == VF_CONST) && (cp->vf == VF_CONST)) {
 				calc_const(cp, j, c, e);
 			} else {
@@ -1218,59 +1249,24 @@ static int expr_next(compile_t* cp, int prev)
 	return c;
 }
 
-/************************************************************
- *
- ************************************************************/
-static int expr_part(compile_t* cp)
-{
-	op_t c;
-	put_c(cp, C_line);
-	put_ushort(cp, cp->ln);
-	c = expr_next(cp, 0);
-	exerrif(cp, !(cp->vf & VF_RESOLV), "bad expr");
-	return c;
-}
-
-static int expr_if(compile_t* cp)
-{
-	op_t c;
-	int k;
-	put_c(cp, C_line);
-	put_ushort(cp, cp->ln);
-	exerrif(cp, (c = lex(cp, 0)) <= 1, "bad for expr");
-	if (c == C_var) {
-		c = lex(cp, 0);
-		exerrif(cp, c != C_symbol, "bad define var");
-		k = add_symbol(cp, cp->sc, cp->nc);
-		add_local(cp, C_var, k, 0);
-		c = lex(cp, 0);
-		exerrif(cp, c != '=', "bad assign local");
-		put_c_int(cp, C_symbol, k);
-		c = expr_next(cp, 0);
-		exerrif(cp, !(cp->vf & VF_RESOLV), "bad expr");
-		put_c(cp, C_store);
-		put_c(cp, C_load);
-	} else {
-		unlex(cp, c);
-		c = expr_next(cp, 0);
-		exerrif(cp, !(cp->vf & VF_RESOLV), "bad expr");
-	}
-	return c;
-}
-
-static int expr_var(compile_t* cp)
+static int expr_vars(compile_t* cp)
 {
 	op_t c;
 	int k;
 	do {
 		c = lex(cp, 0);
-		exerrif(cp, c != C_symbol, "bad define var");
+		exerrif(cp, c != C_symbol, "bad vars");
 		k = add_symbol(cp, cp->sc, cp->nc);
 		add_local(cp, C_var, k, 0);
+		
 		c = lex(cp, 0);
 		if (c == '=') {
 			put_c_int(cp, C_symbol, k);
-			c = expr_part(cp);
+			
+			put_lineno(cp);
+			c = expr_brace(cp, 0);
+			exerrif(cp, !cp->vf, "bad vars");
+			
 			put_c(cp, C_store);
 			put_c(cp, C_pop);
 		}
@@ -1282,9 +1278,13 @@ static int expr_line(compile_t* cp)
 {
 	op_t c;
 	do {
-		c = expr_part(cp);
-		exerrif(cp, !c || !cp->vf, "bad expr");
-		if (c == 1) {
+		
+		put_lineno(cp);
+		c = expr_brace(cp, 0);
+		
+		if (!c || !cp->vf) {
+			;
+		} else if (c == -1) { /* EOF */
 			put_c(cp, C_return);
 		} else if (!(cp->lf & LF_ECHO)) {
 			put_c(cp, C_pop);
@@ -1394,6 +1394,7 @@ static int frame_out(compile_t* cp, cp_frame_t* fp)
  ************************************************************/
 static int stmt_line(compile_t* cp);
 static int stmt_file(compile_t* cp, cp_frame_t* fp);
+static int stmt_block(compile_t* cp);
 
 /************************************************************
  *
@@ -1409,10 +1410,11 @@ static int stmt_Minclude(compile_t* cp)
 	int i, n;
 	
 	i = cp->bn;
-	c = expr_next(cp, 0);
-	exerrif(cp, c != C_crlf, "bad macro");
-	exerrif(cp, cp->vf != VF_CONST, "bad include value");
-	exerrif(cp, cp->bc[i] != C_string, "bad include value");
+	c = expr_brace(cp, 0);
+	exerrif(cp, c != C_crlf, "bad include");
+	exerrif(cp, cp->vf != VF_CONST, "bad include");
+	
+	exerrif(cp, cp->bc[i] != C_string, "bad include");
 	cp->bc[i++] = C_file;
 	fn = cp->bc + i;
 	n = *(int*)fn;
@@ -1483,12 +1485,12 @@ static int stmt_Mdefine(compile_t* cp)
 	exerrif(cp, c != C_symbol, "bad define");
 	k = strtoh(cp->sc, cp->nc);
 	i = cp->bn;
-	c = expr_next(cp, 0);
-	exerrif(cp, c != C_crlf, "bad macro");
+	c = expr_brace(cp, 0);
+	exerrif(cp, c != C_crlf, "bad define");
 	if (cp->bn == i) {
 		put_c(cp, C_null);
 	} else {
-		exerrif(cp, cp->vf != VF_CONST, "bad value for const");
+		exerrif(cp, cp->vf != VF_CONST, "bad define");
 	}
 	add_define(cp, k, cp->bc + i, cp->bn - i);
 	cp->bn = i;
@@ -1507,8 +1509,9 @@ static int stmt_if(compile_t* cp)
 	
 	exerrif(cp, c != '(', "bad if expr");
 	
-	c = expr_if(cp);
-	
+	put_lineno(cp);
+	c = expr_brace(cp, 0);
+	exerrif(cp, !cp->vf, "bad if expr");
 	exerrif(cp, c != ')', "bad if expr");
 	
 	put_c(cp, C_and);
@@ -1524,7 +1527,7 @@ static int stmt_if(compile_t* cp)
 		c = stmt_line(cp);
 		set_offset(cp, e);
 	} else {
-		if ((c > 1) && (c != ';')) {
+		if ((c > 0) && (c != ';')) { /* > EOF, ERR */
 			unlex(cp, c);
 			c = ';';
 		}
@@ -1572,8 +1575,9 @@ static int stmt_foreach(compile_t* cp)
 	put_c_int(cp, C_symbol, k);
 	put_c_int(cp, C_symbol, v);
 	
-	c = expr_part(cp);
-	
+	put_lineno(cp);
+	c = expr_brace(cp, 0);
+	exerrif(cp, !cp->vf, "bad foreach expr");
 	exerrif(cp, c != ')', "bad foreach expr");
 	
 	s = cp->bn;
@@ -1608,9 +1612,10 @@ static int stmt_for(compile_t* cp)
 	c = lex(cp, 0);
 	exerrif(cp, c != '(', "bad for expr");
 	
-	exerrif(cp, (c = lex(cp, 0)) <= 1, "bad for expr");
+	c = lex(cp, 0);
+	exerrif(cp, c <= 0, "bad for expr"); /* <= EOF, ERR */
 	if (c == C_var) {
-		c = expr_var(cp);
+		c = expr_vars(cp);
 	} else {
 		if (c != ';') {
 			unlex(cp, c);
@@ -1620,13 +1625,17 @@ static int stmt_for(compile_t* cp)
 	exerrif(cp, c != ';', "bad for expr");
 	
 	r = s = cp->bn;
-	c = expr_part(cp);
 	
+	put_lineno(cp);
+	c = expr_brace(cp, 0);
+	exerrif(cp, !cp->vf, "bad for expr");
 	exerrif(cp, c != ';', "bad for expr");
+	
 	put_c(cp, C_loop);
 	e = seek_int(cp, 0);
 	
-	exerrif(cp, (c = lex(cp, 0)) <= 1, "bad for expr");
+	c = lex(cp, 0);
+	exerrif(cp, c <= 0, "bad for expr"); /* <= EOF, ERR */
 	if (c != ')') {
 		unlex(cp, c);
 		
@@ -1668,8 +1677,9 @@ static int stmt_while(compile_t* cp)
 	c = lex(cp, 0);
 	exerrif(cp, c != '(', "bad while expr");
 	
-	c = expr_if(cp);
-	
+	put_lineno(cp);
+	c = expr_brace(cp, 0);
+	exerrif(cp, !cp->vf, "bad while expr");
 	exerrif(cp, c != ')', "bad while expr");
 	
 	put_c(cp, C_loop);
@@ -1703,13 +1713,14 @@ static int stmt_do(compile_t* cp)
 	exerrif(cp, c != C_while, "bad do");
 	
 	c = lex(cp, 0);
-	exerrif(cp, c != '(', "bad while expr");
+	exerrif(cp, c != '(', "bad do expr");
 	
 	add_local(cp, C_label, K_continue, cp->bn);
 	
-	c = expr_if(cp);
-	
-	exerrif(cp, c != ')', "bad while expr");
+	put_lineno(cp);
+	c = expr_brace(cp, 0);
+	exerrif(cp, !cp->vf, "bad do expr");
+	exerrif(cp, c != ')', "bad do expr");
 	
 	put_c(cp, C_loop);
 	e = seek_int(cp, 0);
@@ -1737,23 +1748,29 @@ static int stmt_switch(compile_t* cp)
 	c = lex(cp, 0);
 	exerrif(cp, c != '(', "bad switch expr");
 	
-	c = expr_if(cp);
-	
+	put_lineno(cp);
+	c = expr_brace(cp, 0);
+	exerrif(cp, !cp->vf, "bad switch expr");
 	exerrif(cp, c != ')', "bad switch expr");
 	
 	exerrif(cp, lex(cp, LF_LEFT) != '{', "bad switch statement");
 	
 	j = k = 0;
 	do {
-		exerrif(cp, (c = lex(cp, LF_LEFT)) <= 1, "bad switch content");
+		c = lex(cp, LF_LEFT);
+		exerrif(cp, c <= 0, "bad switch content"); /* <= EOF, ERR */
 		if (c == C_case) {
 			if (j) {
 				put_c(cp, C_jump);
 				k = seek_int(cp, k);
 				set_offset(cp, j);
 			}
-			c = expr_part(cp);
+
+			put_lineno(cp);
+			c = expr_brace(cp, 0);
+			exerrif(cp, !cp->vf, "bad case");
 			exerrif(cp, c != ':', "bad case");
+			
 			put_c(cp, C_case);
 			j = seek_int(cp, 0);
 			c = ';';
@@ -1816,7 +1833,7 @@ static int stmt_try(compile_t* cp)
 		c = stmt_line(cp);
 		set_offset(cp, j);
 	} else {
-		if ((c > 1) && (c != ';')) {
+		if ((c > 0) && (c != ';')) { /* > EOF, ERR */
 			unlex(cp, c);
 			c = ';';
 		}
@@ -1829,20 +1846,27 @@ static int stmt_try(compile_t* cp)
 	return c;
 }
 
-static int stmt_function(compile_t* cp)
+static int stmt_function(compile_t* cp, int prev)
 {
 	cp_frame_t fp;
 	op_t c;
 	int k, n, j;
 	
-	put_c(cp, C_jump);
-	j = seek_int(cp, 0);
-	
-	c = lex(cp, 0);
-	exerrif(cp, c != C_symbol, "bad function symbol");
-	
-	k = add_symbol(cp, cp->sc, cp->nc);
-	add_local(cp, C_function, k, cp->bn);
+	if (!prev) {
+		
+		c = lex(cp, 0);
+		exerrif(cp, c != C_symbol, "bad function symbol");
+		
+		k = add_symbol(cp, cp->sc, cp->nc);
+		
+		put_c(cp, C_jump);
+		j = seek_int(cp, 0); /* seek before `C_function` */
+		add_local(cp, C_function, k, cp->bn);
+		
+	} else {
+		put_c_int(cp, C_function, 0);
+		j = seek_int(cp, 0);
+	}
 	
 	frame_in(cp, &fp, AF_FILE | AF_RETURN);
 	
@@ -1864,10 +1888,10 @@ static int stmt_function(compile_t* cp)
 	
 	put_c_int(cp, C_args, n);
 	
-	exerrif(cp, (c = lex(cp, LF_LEFT)) != '{', "bad function statement");
-	unlex(cp, c);
-	
-	c = stmt_line(cp);
+	c = lex(cp, LF_LEFT);
+	exerrif(cp, c != '{', "bad function statement");
+	c = stmt_block(cp);
+	exerrif(cp, c != ';', "bad function statement");
 	
 	put_c(cp, C_null);
 	put_c(cp, C_return);
@@ -1918,9 +1942,8 @@ static int stmt_class(compile_t* cp)
 	}
 	
 	exerrif(cp, c != '{', "bad class statement");
-	unlex(cp, c);
-	
-	c = stmt_line(cp);
+	c = stmt_block(cp);
+	exerrif(cp, c != ';', "bad class statement");
 	
 	put_c(cp, C_this);
 	put_c(cp, C_return);
@@ -1986,7 +2009,7 @@ static int stmt_line(compile_t* cp)
 		c = stmt_class(cp);
 	} else if (c == C_function) {
 		exerrif(cp, !(cp->fp->af & AF_FUNCTION), "bad function");
-		c = stmt_function(cp);
+		c = stmt_function(cp, 0);
 	} else if (c == C_Minclude) {
 		c = stmt_Minclude(cp);
 	} else if (c == C_Mdefine) {
@@ -1999,9 +2022,9 @@ static int stmt_line(compile_t* cp)
 		exerrif(cp, c != ':', "bad label");
 		c = ';';
 	} else if (c == C_var) {
-		c = expr_var(cp);
+		c = expr_vars(cp);
 	} else {
-		if ((c > 1) && (c != ';') && (c != '}')) {
+		if ((c > 0) && (c != ';') && (c != '}')) { /* > EOF, ERR */
 			unlex(cp, c);
 			c = expr_line(cp);
 		}
@@ -2051,7 +2074,7 @@ static int stmt_file(compile_t* cp, cp_frame_t* fp)
 		c = stmt_line(cp);
 	} while (c == ';');
 	
-	if (c != 1) {
+	if (c != -1) { /* != EOF */
 		char *s, *e;
 		s = e = cp->sc;
 		while (((cp->sc - s) < 16) && (s > cp->ss)
