@@ -433,19 +433,6 @@ static int ex_class(exec_t* ex)
 	
 	return -1;
 }
-static int ex_function(exec_t* ex)
-{
-	var_t v;
-	int k;
-	char* j;
-	
-	k = read_int(&ex->bc);
-	j = read_offset(&ex->bc);
-	new_function(&v, j);
-	frame_setLocal(ex, ex->gp, k, &v);
-	
-	return -1;
-}
 static int ex_this(exec_t* ex)
 {
 	var_t v;
@@ -457,6 +444,25 @@ static int ex_this(exec_t* ex)
 	
 	return -1;
 }
+static int ex_function(exec_t* ex)
+{
+	var_t v;
+	int k;
+	char* j;
+	
+	k = read_int(&ex->bc);
+	j = read_offset(&ex->bc);
+	if (k != 0) {
+		new_function(&v, j);
+		frame_setLocal(ex, ex->gp, k, &v);
+	} else {
+		new_function(&v, ex->bc);
+		push_stack(ex, &v);
+		ex->bc = j;
+	}
+	
+	return -1;
+}
 
 /************************************************************
  *
