@@ -9,12 +9,52 @@
 /************************************************************
  *
  ************************************************************/
-static int _file_close(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+typedef struct {
+	FILE* fp;
+	int (*xp)(FILE*);
+} file_t;
+
+int new_file(exec_t* ex, var_t* v, FILE* fp, int (*xp)(FILE*))
 {
-	new_bool(res, fclose(fp) == 0);
+	file_t* ft;
+	ft = (file_t*)alloc_link(ex, sizeof(file_t), file_proto);
+	ft->fp = fp;
+	ft->xp = xp;
+	v->p = file_proto;
+	v->u.p = (void*)ft;
 	return 1;
 }
-static int _file_read(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+
+/************************************************************
+ *
+ ************************************************************/
+void file_clear(exec_t* ex, void* p)
+{
+	file_t* ft = p;
+	if (ft->fp && ft->xp) {
+		(*ft->xp)(ft->fp);
+	}
+	ft->fp = NULL;
+	ft->xp = NULL;
+}
+void file_trace(exec_t* ex, void *p)
+{
+	file_t* ft = p;
+	fprintf(ex->out, "fp:%p, xp:%p\n", ft->fp, ft->xp);
+}
+
+/************************************************************
+ *
+ ************************************************************/
+static int _file_close(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
+{
+	exerrif(ex, !ft->fp || !ft->xp, "can't close the file");
+	new_bool(res, (*ft->xp)(ft->fp) == 0);
+	ft->fp = NULL;
+	ft->xp = NULL;
+	return 1;
+}
+static int _file_read(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	char *s;
 	int n;
@@ -25,7 +65,7 @@ static int _file_read(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 	
 	s = malloc(n + 1);
 	
-	if (fread(s, 1, n, fp) != n) {
+	if (fread(s, 1, n, ft->fp) != n) {
 		free(s);
 		return 0;
 	}
@@ -34,7 +74,7 @@ static int _file_read(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 	
 	return 1;
 }
-static int _file_write(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_write(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	char *s;
 	int n;
@@ -46,29 +86,29 @@ static int _file_write(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 		n = var_toInt(ex, argv + 1);
 	}
 	exerrif(ex, n <= 0,  "bad argument-1");
-	if (fwrite(s, 1, n, fp) != n) {
+	if (fwrite(s, 1, n, ft->fp) != n) {
 		return 0;
 	}
 	new_int(res, n);
 	return 1;
 }
-static int _file_getc(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_getc(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	int c;
 	exerrif(ex, argc != 0,  "bad arguments");
-	if ((c = fgetc(fp)) < 0) {
+	if ((c = fgetc(ft->fp)) < 0) {
 		return 0;
 	}
 	new_char(res, c);
 	return 1;
 }
-static int _file_putc(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_putc(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	int c,i,m=0;
 	exerrif(ex, argc < 1,  "bad arguments");
 	for (i = 0; i < argc; i++) {
 		c = var_toChar(ex, argv + i);
-		if ((c < 0) || (fputc(c, fp) < 0)) {
+		if ((c < 0) || (fputc(c, ft->fp) < 0)) {
 			/* put error */
 			return 0;
 		}
@@ -77,7 +117,7 @@ static int _file_putc(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 	new_int(res, m);
 	return 1;
 }
-static int _file_gets(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_gets(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	char *s;
 	int n, c;
@@ -87,11 +127,11 @@ static int _file_gets(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 	s = malloc(BUFSIZ);
 	n = 0;
 	
-	while ((c = fgetc(fp)) > 0) {
+	while ((c = fgetc(ft->fp)) > 0) {
 		if (c == 0xd) {
-			c = fgetc(fp);
+			c = fgetc(ft->fp);
 			if ((c > 0) && (c != 0xa)) {
-				ungetc(c, fp);
+				ungetc(c, ft->fp);
 			}
 			c = 0xa;
 		}
@@ -111,14 +151,14 @@ static int _file_gets(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 	
 	return 1;
 }
-static int _file_puts(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_puts(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	char *s;
 	int i, n, m = 0;
 	exerrif(ex, argc < 1,  "bad arguments");
 	for (i = 0; i < argc; i++) {
 		if (var_toString(ex, argv + i, &s, &n)) {
-			if (fwrite(s, 1, n, fp) != n) {
+			if (fwrite(s, 1, n, ft->fp) != n) {
 				/* fwrite error */
 				return 0;
 			}
@@ -128,28 +168,28 @@ static int _file_puts(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 	new_int(res, m);
 	return 1;
 }
-static int _file_printf(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_printf(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	char *s;
 	exerrif(ex, argc < 1,  "bad arguments");
 	exerrif(ex, !(s = var_toChars(ex, argv)), "bad argument-0") ;
-	new_int(res, printf_to_file(ex, fp, s, argv + 1, argc - 1));
+	new_int(res, printf_to_file(ex, ft->fp, s, argv + 1, argc - 1));
 	return 1;
 }
-static int _file_tell(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_tell(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	exerrif(ex, argc != 0,  "bad arguments");
-	new_int(res, ftell(fp));
+	new_int(res, ftell(ft->fp));
 	return 1;
 }
-static int _file_rewind(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_rewind(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	exerrif(ex, argc != 0,  "bad arguments");
-	rewind(fp);
+	rewind(ft->fp);
 	new_bool(res, 1);
 	return 1;
 }
-static int _file_seek(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
+static int _file_seek(exec_t* ex, file_t* ft, int argc, var_t* argv, var_t* res)
 {
 	int i = 0, w = SEEK_SET;
 	exerrif(ex, argc < 1,  "bad arguments");
@@ -164,7 +204,7 @@ static int _file_seek(exec_t* ex, FILE* fp, int argc, var_t* argv, var_t* res)
 			return 0;
 		}
 	}
-	new_bool(res, fseek(fp, i, w) == 0);
+	new_bool(res, fseek(ft->fp, i, w) == 0);
 	return 1;
 }
 
