@@ -6,13 +6,100 @@
 #include "_sym.h"
 #include "_ext.h"
 
+/************************************************************
+ *
+ ************************************************************/
 /*
 sleep
 alarm
 fork
-exec (execl, execve)
 */
 
+/************************************************************
+ *
+ ************************************************************/
+#include <sys/wait.h>
+
+static int _global_exec(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	var_t *ref;
+	char buf[BUFSIZ + 2];
+	char *cmd, *s;
+	int pid, rc, sz, n, i;
+	int out[2], in[2];
+	
+	exerrif(ex, argc < 1, "bad arguments");
+	exerrif(ex, !(cmd = var_toChars(ex, argv)), "bad commands");
+	
+	ref = NULL;
+	s = NULL;
+	n = 0;
+	for (i = 1; i < argc; i++) {
+		if (is_refer_var(argv + i)) {
+			if (ref != NULL) {
+				fprintf(ex->err, "bad argument %d\n", i);
+				return 0;
+			}
+			ref = argv + i;
+		} else {
+			if ((s != NULL) || !var_toString(ex, argv + i, &s, &n)) {
+				fprintf(ex->err, "bad argument %d\n", i);
+				return 0;
+			}
+		}
+	}
+	
+	assert(pipe(out) == 0);
+	
+	if ((pid = fork()) == 0) {
+		close(out[0]);
+		if (s && n) {
+			assert(pipe(in) == 0);
+			write(in[1], s, n);
+			close(in[1]);
+			dup2(in[0], 0);
+		}
+		dup2(out[1], 1);
+		dup2(out[1], 2);
+		execlp("sh", "sh", "-c", cmd, NULL);
+		exit(-1);
+	}
+	
+	close(out[1]);
+	if (waitpid(pid, &rc, 0) != pid) {
+		rc = -1; /* waitpid error */
+	} else if (!WIFEXITED(rc)) {
+		rc = -1; /* not exited */
+	} else {
+		rc = WEXITSTATUS(rc);
+	}
+	if (ref != NULL) {
+		new_int(res, rc);
+		refer_set(ex, ref, res);
+	}
+	
+	s = NULL;
+	n = 0;
+	if (rc >= 0) {
+		sz = BUFSIZ;
+		while ((sz >= BUFSIZ) && ((sz = read(out[0], buf, BUFSIZ)) > 0)) {
+			s = realloc(s, n + sz + 1);
+			memcpy(s + n, buf, sz);
+			n += sz;
+		}
+	}
+	close(out[0]);
+	
+	if (s) {
+		s[n] = 0;
+		new_string_with(ex, res, s, n);
+	} else {
+		new_null(res);
+	}
+	
+	return 1;
+}
+
 /************************************************************
  *
  ************************************************************/
@@ -60,16 +147,7 @@ static int _global_popen(exec_t* ex, int argc, var_t* argv, var_t* res)
 	exerrif(ex, !(cmd = var_toChars(ex, argv)), "bad argument-0");
 	exerrif(ex, (argc >= 2) && !(mode = var_toChars(ex, argv)), "bad argument-1");
 	exerrif(ex, !(fp = popen(cmd, mode)), "popen error");
-	new_file(res, fp);
-	return 1;
-}
-
-static int _global_pclose(exec_t* ex, int argc, var_t* argv, var_t* res)
-{
-	exerrif(ex, argc != 1, "bad arguments");
-	exerrif(ex, argv->p != file_proto, "bad argument-0");
-	exerrif(ex, pclose((FILE*)argv->u.p) != 0, "pclose error");
-	new_bool(res, 1);
+	new_file(ex, res, fp, pclose);
 	return 1;
 }
 
@@ -249,7 +327,7 @@ static int _global_tracef(exec_t* ex, int argc, var_t* argv, var_t* res)
 	exerrif(ex, argc < 1,  "bad arguments");
 	if (argc >= 2) {
 		if ((argv + 1)->p == file_proto) {
-			fp = (argv + 1)->u.p;
+			fp = *(FILE**)((argv + 1)->u.p);
 		} else {
 			exerrif(ex, !(fn = var_toChars(ex, argv + 1)), "bad argument 1");
 			exerrif(ex, !(fp = fopen(fn, "w")), "fopen error");
@@ -672,7 +750,7 @@ static int _global_fopen(exec_t* ex, int argc, var_t* argv, var_t* res)
 	exerrif(ex, (argc >= 2) && !(mode = var_toChars(ex, argv + 1)), "bad argument-1");
 	
 	exerrif(ex, !(fp = fopen(path, mode)), "fopen error");
-	new_file(res, fp);
+	new_file(ex, res, fp, fclose);
 	return 1;
 }
 static int _global_tmpfile(exec_t* ex, int argc, var_t* argv, var_t* res)
@@ -680,7 +758,7 @@ static int _global_tmpfile(exec_t* ex, int argc, var_t* argv, var_t* res)
 	FILE* fp;
 	exerrif(ex, argc != 0,  "bad arguments");
 	assert((fp = tmpfile()) != NULL);
-	new_file(res, fp);
+	new_file(ex, res, fp, fclose);
 	return 1;
 }
 static int _global_tempnam(exec_t* ex, int argc, var_t* argv, var_t* res)
@@ -695,11 +773,12 @@ static int _global_tempnam(exec_t* ex, int argc, var_t* argv, var_t* res)
 	if (argc >= 2) s2 = var_toChars(ex, argv + 1);
 	if (argc >= 3) s3 = var_toChars(ex, argv + 2);
 	if (!s1) s1 = P_tmpdir;
+	if (!s1) s1 = "";
 	if (!s2) s2 = "";
 	if (!s3) s3 = "";
 	
 	exerrif(ex,
-		!s1 || !*s1 || (stat(s1, &st) != 0) || !(st.st_mode & S_IFDIR),
+		*s1 && ((stat(s1, &st) != 0) || !(st.st_mode & S_IFDIR)),
 		"bad directory");
 	
 	exerrif(ex,
@@ -707,9 +786,10 @@ static int _global_tempnam(exec_t* ex, int argc, var_t* argv, var_t* res)
 		"too large file-name");
 	
 	s1 = strchr(strcpy(buf, s1), 0);
-	if (*(s1 - 1) != '/') {
+	if (*buf && (*(s1 - 1) != '/')) {
 		*s1++ = '/';
 	}
+	
 	if (*s2) {
 		s1 = strchr(strcpy(s1, s2), 0);
 	}
@@ -1070,7 +1150,7 @@ static int _global_dbopen(exec_t* ex, int argc, var_t* argv, var_t* res)
 	exerrif(ex, !(path = var_toChars(ex, argv)), "bad argument-0");
 	exerrif(ex, (argc >= 2) && !(mode = var_toChars(ex, argv)), "bad argument-1");
 	
-	return dbm_open(ex, res, path, mode);
+	return new_dbm(ex, res, path, mode);
 }
 
 /************************************************************
@@ -1084,7 +1164,7 @@ static int _global_xmlopen(exec_t* ex, int argc, var_t* argv, var_t* res)
 	exerrif(ex, !(path = var_toChars(ex, argv)), "bad argument-0");
 	exerrif(ex, (argc >= 2) && !(mode = var_toChars(ex, argv)), "bad argument-1");
 	
-	return xml_open(ex, res, path, mode);
+	return new_xml_from_file(ex, res, path, mode);
 }
 
 /************************************************************
@@ -1143,15 +1223,15 @@ static int _global_date(exec_t* ex, int argc, var_t* argv, var_t* res)
 {
 	char dst[BUFSIZ];
 	char *s;
-	int t,n;
+	int t, n;
 	t = vars_toTime(ex, argc - 1, argv + 1, 0);
 	if (argc < 1) {
 		s = "f";
 		n = 1;
 	} else {
-		exerrif(ex, !var_toString(ex, argv, &s, &n) || (n >= 256),  "bad argument-0");
+		exerrif(ex, !(s = var_toChars(ex, argv)),  "bad argument-0");
 	}
-	n = timetostr(dst, s, n, t, 0);
+	n = timetostr(dst, s, t, 0);
 	if (!n) {
 		return 0;
 	}
@@ -1168,9 +1248,9 @@ static int _global_gmdate(exec_t* ex, int argc, var_t* argv, var_t* res)
 		s = "f";
 		n = 1;
 	} else {
-		exerrif(ex, !var_toString(ex, argv, &s, &n) || (n >= 256),  "bad argument-0");
+		exerrif(ex, !(s = var_toChars(ex, argv)),  "bad argument-0");
 	}
-	n = timetostr(dst, s, n, t, 1);
+	n = timetostr(dst, s, t, 1);
 	if (!n) {
 		return 0;
 	}
@@ -1178,6 +1258,198 @@ static int _global_gmdate(exec_t* ex, int argc, var_t* argv, var_t* res)
 	return 1;
 }
 
+/************************************************************
+ * HTTP / NETWORK
+ ************************************************************/
+static int _global_putheader(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	char *s;
+	int i,n,m=0;
+	exerrif(ex, !ex->hh, "no http header");
+	exerrif(ex, argc < 1,  "bad arguments");
+	for (i = 0; i < argc; i++) {
+		if (var_toString(ex, argv + i, &s, &n)) {
+			while ((n > 0) && isspace(s[n - 1])) n--;
+			if (n > 0) {
+				m += fprintf(ex->hh, "%.*s\n", n, s);
+			}
+		}
+	}
+	new_int(res, m);
+	return 1;
+}
+
+static int _global_setcookie(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	char buf[BUFSIZ];
+	char *k, *v;
+	int d, n;
+	
+	exerrif(ex, !ex->hh, "no http header");
+	exerrif(ex, argc < 1,  "bad arguments");
+	exerrif(ex, !(k = var_toChars(ex, argv)), "bad key");
+	
+	if (!var_toString(ex, argv + 1, &v, &n)) {
+		v = NULL;
+	} else {
+		encode(&v, v, n, 0);
+	}
+	
+	d = (argc >= 3) ? var_toInt(ex, argv + 2) : 0;
+	if (d) {
+		timetostr(buf, "e", time(NULL) + (d * 86400), 1);
+	}
+	
+	if (!v) {
+		n = fprintf(ex->hh, "Set-Cookie: %s=\n", k);
+	} else if (!d) {
+		n = fprintf(ex->hh, "Set-Cookie: %s=%s\n", k, v);
+	} else {
+		n = fprintf(ex->hh, "Set-Cookie: %s=%s; expires=%s\n", k, v, buf);
+	}
+	if (v) free(v);
+	
+	new_int(res, n);
+	
+	return 1;
+}
+
+/************************************************************
+ * 
+ ************************************************************/
+static int _global_sendmail(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	char *to, *data;
+	
+	data = NULL;
+	
+	exerrif(ex, argc < 1, "bad arguments");
+	exerrif(ex, !(to = var_toChars(ex, argv)), "bad argument 0");
+	exerrif(ex, (argc >= 2) && !(data = var_toChars(ex, argv + 1)), "bad argument 1");
+	
+	new_int(res, sendmail(to, data, ex->err));
+	
+	return 1;
+}
+
+/************************************************************
+ * 
+ ************************************************************/
+static int _global_popmail(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	char *u, *s;
+	int n;
+	FILE* fp;
+	
+	s = NULL; /* password */
+	
+	exerrif(ex, (argc < 1) || !(u = var_toChars(ex, argv)), "bad arguments");
+	exerrif(ex, (argc >= 2) && !(s = var_toChars(ex, argv + 1)), "bad password");
+	
+	fp = tmpfile();
+	if (!popmail(u, s, fp, ex->err)) {
+		fclose(fp);
+		return 0;
+	}
+	
+	n = ftell(fp);
+	rewind(fp);
+	s = calloc(n + 1, 1);
+	fread(s, 1, n, fp);
+	fclose(fp);
+	new_string_with(ex, res, s, n);
+	
+	return 1;
+}
+
+/************************************************************
+ * 
+ ************************************************************/
+static int _global_ftpsite(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	char *u, *p, *cmd, *s;
+	
+	p = cmd = NULL;
+	
+	exerrif(ex, argc < 2, "bad arguments");
+	exerrif(ex, !(u = var_toChars(ex, argv)), "bad argument 0");
+	
+	argc--;
+	argv++;
+	for (; argc-- > 0; argv++) {
+		exerrif(ex, !(s = var_toChars(ex, argv)), "bad argument");
+		if (!cmd) cmd = s;
+		else if (!p) {
+			p = cmd;
+			cmd = s;
+		}
+	}
+	
+	new_int(res, ftpsite(u, p, cmd, ex->err));
+	
+	return 1;
+}
+
+/************************************************************
+ * 
+ ************************************************************/
+static int _global_httpget(exec_t* ex, int argc, var_t* argv, var_t* res)
+{
+	var_t *ref;
+	char *url, *head, *data, *s;
+	int n, j;
+	FILE* fp;
+	char buf[BUFSIZ];
+	
+	ref = NULL;
+	head = data = NULL;
+	
+	exerrif(ex, argc < 1, "bad arguments");
+	exerrif(ex, !(url = var_toChars(ex, argv)), "bad argument 0");
+	
+	argc--;
+	argv++;
+	for (; argc-- > 0; argv++) {
+		if (is_refer_var(argv)) {
+			ref = argv;
+		} else {
+			exerrif(ex, !(s = var_toChars(ex, argv)), "bad header string");
+			if (!head) head = s;
+			if (!data) data = s;
+		}
+	}
+	
+	fp = tmpfile();
+	if (!httpget(url, head, data, fp, ex->err)) {
+		fclose(fp);
+		return 0;
+	}
+	
+	j = ftell(fp);
+	rewind(fp);
+	while (fgets(buf, BUFSIZ, fp)) {
+		if (iscrlf(*buf)) {
+			break;
+		}
+	}
+	if (ref) {
+		n = ftell(fp);
+		rewind(fp);
+		s = calloc(n + 1, 1);
+		fread(s, 1, n, fp);
+		new_string_with(ex, res, s, n);
+		refer_set(ex, ref, res);
+	}
+	n = j - ftell(fp);
+	s = calloc(n + 1, 1);
+	fread(s, 1, n, fp);
+	new_string_with(ex, res, s, n);
+	
+	fclose(fp);
+	
+	return 1;
+}
+
 /************************************************************
  *
  ************************************************************/
