@@ -16,6 +16,7 @@ void usage(char *cmd)
 
 int main(int argc, char *argv[])
 {
+	exec_t ex;
 	char buf[BUFSIZ];
 	char *s, *o;
 	int r, i, c, e;
@@ -46,24 +47,26 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 	
+	exec_init(&ex);
 	if (o) {
 		s = argv[i++];
-		r = compile_file_to_file(s, o);
+		r = compile_file_to_file(&ex, s, o);
 	} else if (c) {
 		s = argv[i++];
 		o = strrchr(strcpy(buf, s), '.');
 		if (!o) o = strchr(buf, 0);
 		strcpy(o, ".jsc");
-		r = compile_file_to_file(s, buf);
+		r = compile_file_to_file(&ex, s, buf);
 	} else if (e) {
 		s = argv[i++];
-		r = exec_string(s, strlen(s));
+		r = exec_code(&ex, s, strlen(s));
 	} else {
 		do {
 			s = argv[i++];
-			r = exec_source(s);
+			r = exec_file(&ex, s);
 		} while ((r == 0) && (i < argc));
 	}
+	exec_clear(&ex);
 	
 	return r;
 }
