@@ -270,8 +270,16 @@ int string_toString(exec_t* ex, var_t* v, char **s, int *n)
 int string_toNumber(exec_t* ex, var_t* v, double *f, int *i)
 {
 	string_t* str = v->u.p;
+	char *e;
 	if (str->s && str->n) {
-		return strtonum(str->s, str->n, f, i);
+		*f = *i = strtol(str->s, &e, 0);
+		if ((e < (str->s + str->n)) && *e && strchr(".Ee", *e)) {
+			*i = *f = strtod(str->s, &e);
+			return 2;
+		}
+		if (e > str->s) {
+			return 1;
+		}
 	}
 	*f = *i = 0;
 	return 0;
