@@ -122,13 +122,13 @@ int strtotime(char *src, int len, int g)
 		if (isalpha(*(arr[i].s))) {
 			if (M <= 0) {
 				for (j=0; j < 12; j++) {
-					if (!strnncasecmp(arr[i].s, 3, g_mons[j], 3)) {
+					if (!strncasecmp(arr[i].s, g_mons[j], 3)) {
 						M = j + 1;
 						break;
 					}
 				}
 			} else {
-				if (!strnncasecmp(arr[i].s, 3, "gmt", 3)) {
+				if (!strncasecmp(arr[i].s, "gmt", 3)) {
 					g = 1;
 				}
 			}
@@ -154,7 +154,7 @@ int strtotime(char *src, int len, int g)
 	return datetotime(Y, M, D, H, I, S, g);
 }
 
-int timetostr(char dst[BUFSIZ], char *fmt, int n, int t, int g)
+int timetostr(char dst[BUFSIZ], char *fmt, int t, int g)
 {
 	time_t tv;
 	struct tm *tm;
