@@ -6,38 +6,6 @@
 /************************************************************
  *
  ************************************************************/
-char* tmpbuf()
-{
-	static char p[16][256];
-	static int i = 0;
-	if (i>=16) i=0;
-	return p[i++];
-}
-
-/************************************************************
- *
- ************************************************************/
-char* strndupok(char *s, int n)
-{
-	char *p = malloc(n + 1);
-	/* strncpy */
-	memcpy(p, s, n);
-	p[n] = 0;
-	return p;
-}
-int sputsn(char *d, char *s, int n)
-{
-	memcpy(d, s, n);
-	/* d[n] = 0; */
-	return n;
-}
-int sputs(char *d, char *s)
-{
-	int n = strlen(s);
-	memcpy(d, s, n);
-	/* d[n] = 0; */
-	return n;
-}
 int strnncmp(char *s1, int n1, char* s2, int n2)
 {
 	int i,r;
@@ -55,23 +23,6 @@ int strrnncmp(char *s1, int n1, char* s2, int n2)
 	}
 	return 0;
 }
-int strnncasecmp(char *s1, int n1, char* s2, int n2)
-{
-	int i,r;
-	for (i=0; (i < n1) && (i < n2); i++) {
-		if ((r = tolower(s1[i]) - tolower(s2[i])) != 0) return r;
-	}
-	return (n1 - n2);
-}
-int strrnncasecmp(char *s1, int n1, char* s2, int n2)
-{
-	int i,r;
-	if ((r = n1 - n2) != 0) return r;
-	for (i=0; i < n1; i++) {
-		if ((r = tolower(s1[i]) - tolower(s2[i])) != 0) return r;
-	}
-	return 0;
-}
 int strtoh(char *s, int n)
 {
 	int i, c, k = 0;
@@ -80,27 +31,17 @@ int strtoh(char *s, int n)
 	}
 	return k;
 }
-int strtonum(char *s, int n, double *f, int *i)
-{
-	char *e;
-	*f = *i = strtol(s, &e, 0);
-	if ((e < (s + n)) && *e && strchr(".Ee", *e)) {
-		*i = *f = strtod(s, &e);
-		return 2;
-	}
-	return (e > s);
-}
-int isoctal(int c)
-{
-	return (c >= '0' && c <= '7');
-}
 int iscrlf(int c)
 {
 	return ((c == 0xa) || (c == 0xd));
 }
-int istvfs(int c)
+
+/***************************************************************
+ *
+ ***************************************************************/
+int isoctal(int c)
 {
-	return ((c==0x9) || (c==0xb) || (c==0xc) || (c==0x20));
+	return (c >= '0' && c <= '7');
 }
 int itoX(int c)
 {
@@ -112,10 +53,6 @@ int xtoi(int c)
 {
 	return (isalpha(c) ? (10 + tolower(c) - 'a') : (isdigit(c) ? (c - '0') : 0));
 }
-int ischar(int c, char *s)
-{
-	return (c && (strchr(s, c) != NULL));
-}
 int c_to_meta(int* c, int q)
 {
 	if ((*c == q) || (*c == '\\')) {
@@ -192,27 +129,3 @@ int entity_to_c(char **s)
 	if (**s == ';') (*s)++;
 	return c;
 }
-int vmclose(char *mp, int sz, FILE* fp)
-{
-	int r1, r2;
-	r1 = munmap(mp, sz) == 0;
-	r2 = fclose(fp) == 0;
-	return (r1 && r2);
-}
-char* vmopen(char *fn, int *sz, FILE **fp)
-{
-	char *mp;
-	if (!(*fp = fopen(fn, "r"))) {
-		return NULL;
-	}
-	fseek(*fp, 0, SEEK_END);
-	*sz = ftell(*fp);
-	rewind(*fp);
-	mp = mmap(0, *sz, PROT_READ, MAP_SHARED, fileno(*fp), 0);
-	if (mp == MAP_FAILED) {
-		fclose(*fp);
-		return NULL;
-	}
-	return mp;
-}
-
