@@ -1323,7 +1323,7 @@ static int format_range(char *xs, int pos, int len,
 					dir++; /* bad term key */
 					sz += fprintf(fp, "bad key: %.*s\n", keylen(k), k);
 				} else {
-					while ((tag > pre) && istvfs(*tag)) tag--;
+					while ((tag > pre) && isspace(*tag) && !iscrlf(*tag)) tag--;
 					while ((next < end) && iscrlf(*next)) next++;
 					if ((spn = tag - pre) > 0) {
 						if (type == 'f') { /* for */
@@ -1348,7 +1348,7 @@ static int format_range(char *xs, int pos, int len,
 			continue;
 		} else if (t) {
 			if (dir++ <= 0) {
-				while ((tag > pre) && istvfs(*tag)) tag--;
+				while ((tag > pre) && isspace(*tag) && !iscrlf(*tag)) tag--;
 				while ((next < end) && iscrlf(*next)) next++;
 				if (tag > pre) {
 					sz += fwrite(pre, 1, tag - pre, fp);
@@ -1565,9 +1565,10 @@ typedef struct {
 	char *f;
 } xml_t;
 
-int xml_create(exec_t* ex, var_t* v, char *s, int n, char *f)
+int new_xml_from_string(exec_t* ex, var_t* v, char *s, int n, char *f)
 {
 	xml_t *x;
+	v->p = NULL;
 	x = (xml_t*)alloc_link(ex, sizeof(xml_t), xml_proto);
 	if (f) {
 		x->f = strdup(f);
@@ -1578,7 +1579,8 @@ int xml_create(exec_t* ex, var_t* v, char *s, int n, char *f)
 	v->u.p = (void*)x;
 	return 1;
 }
-int xml_open(exec_t* ex, var_t* v, char *filename, char* mode)
+
+int new_xml_from_file(exec_t* ex, var_t* v, char *filename, char* mode)
 {
 	char *s;
 	int n, w;
@@ -1592,8 +1594,7 @@ int xml_open(exec_t* ex, var_t* v, char *filename, char* mode)
 		n = 0;
 		assert((s = alloc_stream(BUFSIZ)) != NULL);
 	}
-	xml_create(ex, v, s, n, w ? filename : NULL);
-	return 1;
+	return new_xml_from_string(ex, v, s, n, w ? filename : NULL);
 }
 
 /************************************************************
@@ -1646,7 +1647,7 @@ static int _xml_write(exec_t* ex, xml_t* x, int argc, var_t* argv, var_t* res)
 	exerrif(ex, !x->s, "bad xml");
 	if (argc >= 1) {
 		if (argv->p == file_proto) {
-			fp = argv->u.p;
+			fp = *(FILE**)(argv->u.p);
 		} else {
 			exerrif(ex, !(fn = var_toChars(ex, argv)), "bad argument 0");
 			exerrif(ex, !(fp = fopen(fn, "w")), "fopen error");
@@ -1773,7 +1774,7 @@ static int _xml_select(exec_t* ex, xml_t* x, int argc, var_t* argv, var_t* res)
 	if (!select_stream(x->s, x->n, path, sortby, &s, &n)) {
 		return 0;
 	}
-	xml_create(ex, res, s, n, NULL);
+	new_xml_from_string(ex, res, s, n, NULL);
 	return 1;
 }
 static int _xml_delete(exec_t* ex, xml_t* x, int argc, var_t* argv, var_t* res)
@@ -1814,7 +1815,7 @@ static int _xml_formatf(exec_t* ex, xml_t* x, int argc, var_t* argv, var_t* res)
 	exerrif(ex, (argc < 1) || !(path = var_toChars(ex, argv)), "bad arguments") ;
 	if (argc >= 2) {
 		if ((argv + 1)->p == file_proto) {
-			fp = (argv + 1)->u.p;
+			fp = *(FILE**)((argv + 1)->u.p);
 		} else {
 			exerrif(ex, !(fn = var_toChars(ex, argv + 1)), "bad argument 1");
 			exerrif(ex, !(fp = fopen(fn, "w")), "fopen error");
